@@ -26,9 +26,7 @@
 #include "everything.h"
 #include "constants.h"
 
-#undef TIMING_RUN
-#undef BIG_STUBS
-#define NUM_RUNS 1000
+#define NUM_RUNS 1
 
 #ifdef TIMING_RUN
 #  define d_print(a)
@@ -39,7 +37,12 @@
 extern CORBA_ORB global_orb;
 gboolean         in_proc;
 
-void
+/* Ugly- but hey */
+#define _IN_CLIENT_
+#include "server.c"
+#undef  _IN_CLIENT_
+
+static void
 testConst (void)
 {
 	d_print ("Testing constants...\n");
@@ -53,24 +56,26 @@ testConst (void)
 	g_assert (test_FAVORITE_COLOUR==test_red);
 }
 
-
-void
+static void
 testAttribute (test_TestFactory factory, 
 	       CORBA_Environment *ev)
 {  
 	test_BasicServer objref;
-	CORBA_char *val;
-	CORBA_long lval;
+	CORBA_char      *val;
+	CORBA_long       lval;
+
 	d_print ("Testing attributes...\n");
 	objref = test_TestFactory_getBasicServer (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	g_assert (objref != CORBA_OBJECT_NIL);
 	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
 	val = test_BasicServer__get_foo (objref, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);  
 	g_assert (strcmp (val, constants_STRING_RETN)==0);	
 	CORBA_free (val);
+
 	test_BasicServer__set_foo (objref, constants_STRING_IN, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
@@ -82,7 +87,7 @@ testAttribute (test_TestFactory factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testString (test_TestFactory   factory, 
 	    CORBA_Environment *ev)
 {
@@ -115,80 +120,92 @@ testString (test_TestFactory   factory,
 }
 
 
-void
+static void
 testLong (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
-  test_BasicServer objref;
-  CORBA_long inArg, inoutArg, outArg, retn;
-  d_print ("Testing longs...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = constants_LONG_IN;
-  inoutArg = constants_LONG_INOUT_IN;
-  retn = test_BasicServer_opLong (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == constants_LONG_IN);
-  g_assert (inoutArg == constants_LONG_INOUT_OUT);
-  g_assert (outArg == constants_LONG_OUT);
-  g_assert (retn == constants_LONG_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	test_BasicServer objref;
+	CORBA_long inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing longs...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	inArg = constants_LONG_IN;
+	inoutArg = constants_LONG_INOUT_IN;
+	retn = test_BasicServer_opLong (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == constants_LONG_IN);
+	g_assert (inoutArg == constants_LONG_INOUT_OUT);
+	g_assert (outArg == constants_LONG_OUT);
+	g_assert (retn == constants_LONG_RETN);
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testLongLong (test_TestFactory factory, 
-		  CORBA_Environment *ev)
-{
-  test_BasicServer objref;
-  CORBA_long_long inArg, inoutArg, outArg, retn;
-  d_print ("Testing long longs...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = constants_LONG_LONG_IN;
-  inoutArg = constants_LONG_LONG_INOUT_IN;
-  retn = test_BasicServer_opLongLong (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == constants_LONG_LONG_IN);
-  g_assert (inoutArg == constants_LONG_LONG_INOUT_OUT);
-  g_assert (outArg == constants_LONG_LONG_OUT);
-  g_assert (retn == constants_LONG_LONG_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-}
-
-void testEnum (test_TestFactory factory, 
+static void
+testLongLong (test_TestFactory   factory, 
 	      CORBA_Environment *ev)
 {
-  test_BasicServer objref;
-  test_AnEnum inArg, inoutArg, outArg, retn;
-  d_print ("Testing enums...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = test_ENUM_IN;
-  inoutArg = test_ENUM_INOUT_IN;
-  retn = test_BasicServer_opEnum (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == test_ENUM_IN);
-  g_assert (inoutArg == test_ENUM_INOUT_OUT);
-  g_assert (outArg == test_ENUM_OUT);
-  g_assert (retn == test_ENUM_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	test_BasicServer objref;
+	CORBA_long_long inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing long longs...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	inArg = constants_LONG_LONG_IN;
+	inoutArg = constants_LONG_LONG_INOUT_IN;
+	retn = test_BasicServer_opLongLong (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == constants_LONG_LONG_IN);
+	g_assert (inoutArg == constants_LONG_LONG_INOUT_OUT);
+	g_assert (outArg == constants_LONG_LONG_OUT);
+	g_assert (retn == constants_LONG_LONG_RETN);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
+testEnum (test_TestFactory   factory, 
+	  CORBA_Environment *ev)
+{
+	test_BasicServer objref;
+	test_AnEnum inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing enums...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	inArg = test_ENUM_IN;
+	inoutArg = test_ENUM_INOUT_IN;
+	retn = test_BasicServer_opEnum (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == test_ENUM_IN);
+	g_assert (inoutArg == test_ENUM_INOUT_OUT);
+	g_assert (outArg == test_ENUM_OUT);
+	g_assert (retn == test_ENUM_RETN);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
 testException (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
 	test_BasicServer    objref;
 	test_TestException *ex;
-	CORBA_Environment  *cpyev;
+	CORBA_Environment  *cpyev, *rev = ev;
 
 	d_print ("Testing exceptions...\n");
 
 	objref = test_TestFactory_getBasicServer (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
+	test_BasicServer_opException (CORBA_OBJECT_NIL, ev);
+	g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+	g_assert (strcmp (CORBA_exception_id (ev), ex_CORBA_INV_OBJREF) == 0);
+	CORBA_exception_free (ev);
+
 	test_BasicServer_opException (objref, ev);
   
 	g_assert (ev->_major == CORBA_USER_EXCEPTION);
@@ -216,11 +233,11 @@ testException (test_TestFactory   factory,
 
 	CORBA_free (cpyev);
 
-	CORBA_Object_release (objref, ev);  
-	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (objref, rev);
+	g_assert (rev->_major == CORBA_NO_EXCEPTION);
 }
 
-gboolean
+static gboolean
 find_tc (CORBA_sequence_CORBA_TypeCode *tcs, 
 	 const char                    *repo_id)
 {
@@ -233,7 +250,7 @@ find_tc (CORBA_sequence_CORBA_TypeCode *tcs,
 	return FALSE;
 }
 
-void
+static void
 testIInterface (test_TestFactory   factory, 
 		CORBA_Environment *ev)
 {
@@ -304,7 +321,7 @@ testIInterface (test_TestFactory   factory,
 	CORBA_free (tcs);
 }
 
-void
+static void
 testFixedLengthStruct (test_TestFactory   factory, 
 		       CORBA_Environment *ev)
 {
@@ -336,8 +353,9 @@ testFixedLengthStruct (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthStruct (test_TestFactory factory, 
-			      CORBA_Environment *ev)
+static void
+testVariableLengthStruct (test_TestFactory   factory, 
+			  CORBA_Environment *ev)
 {
   test_StructServer objref;
   test_VariableLengthStruct inArg, inoutArg, *outArg, *retn;
@@ -364,8 +382,9 @@ void testVariableLengthStruct (test_TestFactory factory,
 }
 
 
-void testCompoundStruct (test_TestFactory factory, 
-			CORBA_Environment *ev)
+static void
+testCompoundStruct (test_TestFactory   factory, 
+		    CORBA_Environment *ev)
 {
   test_StructServer objref;
   test_CompoundStruct inArg, inoutArg, *outArg, *retn;
@@ -392,7 +411,28 @@ void testCompoundStruct (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
+testObjectStruct (test_TestFactory   factory, 
+		  CORBA_Environment *ev)
+{
+  test_StructServer objref;
+  test_ObjectStruct inArg;
+
+  d_print ("Testing object structs...\n");
+  objref = test_TestFactory_getStructServer (factory, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  inArg.serv = objref;
+
+  test_StructServer_opObjectStruct (objref, &inArg, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  CORBA_Object_release (objref, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+}
+
+static void
 testStructAny (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
@@ -415,8 +455,9 @@ testStructAny (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testUnboundedSequence (test_TestFactory factory, 
-			   CORBA_Environment *ev)
+static void
+testUnboundedSequence (test_TestFactory   factory, 
+		       CORBA_Environment *ev)
 {
   test_SequenceServer objref;
   test_StrSeq inArg, inoutArg, *outArg, *retn;
@@ -444,21 +485,17 @@ void testUnboundedSequence (test_TestFactory factory,
   retn = test_SequenceServer_opStrSeq (objref, &inArg, &inoutArg, &outArg, ev);
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
   
-  for (i=0;i<inArg._length;i++){
+  for (i=0;i<inArg._length;i++)
 	g_assert (strcmp (inArg._buffer[i], constants_SEQ_STRING_IN[i]) == 0);
-  }
 
-  for (i=0;i<inoutArg._length;i++){
+  for (i=0;i<inoutArg._length;i++)
 	g_assert (strcmp (inoutArg._buffer[i], constants_SEQ_STRING_INOUT_OUT[i]) == 0);
-  }
 
-  for (i=0;i<outArg->_length;i++){
+  for (i=0;i<outArg->_length;i++)
 	g_assert (strcmp (outArg->_buffer[i], constants_SEQ_STRING_OUT[i]) == 0);
-  }
 
-  for (i=0;i<retn->_length;i++){
+  for (i=0;i<retn->_length;i++)
 	g_assert (strcmp (retn->_buffer[i], constants_SEQ_STRING_RETN[i]) == 0);
-  }
 
   CORBA_free (inArg._buffer);
   CORBA_free (inoutArg._buffer);
@@ -468,7 +505,7 @@ void testUnboundedSequence (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testBoundedSequence (test_TestFactory   factory, 
 		     CORBA_Environment *ev)
 {
@@ -526,8 +563,9 @@ testBoundedSequence (test_TestFactory   factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testFixedLengthUnion (test_TestFactory factory, 
-			  CORBA_Environment *ev)
+static void
+testFixedLengthUnion (test_TestFactory   factory, 
+		      CORBA_Environment *ev)
 {
   test_UnionServer objref;
   test_FixedLengthUnion inArg, inoutArg, outArg, retn;
@@ -558,8 +596,9 @@ void testFixedLengthUnion (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthUnion (test_TestFactory factory, 
-			     CORBA_Environment *ev)
+static void
+testVariableLengthUnion (test_TestFactory   factory, 
+			 CORBA_Environment *ev)
 {
   test_UnionServer objref;
   test_VariableLengthUnion inArg, inoutArg, *outArg, *retn;
@@ -592,8 +631,9 @@ void testVariableLengthUnion (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testFixedLengthArray (test_TestFactory factory, 
-			  CORBA_Environment *ev)
+static void
+testFixedLengthArray (test_TestFactory   factory, 
+		      CORBA_Environment *ev)
 {
   
   test_ArrayServer objref;
@@ -627,8 +667,9 @@ void testFixedLengthArray (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthArray (test_TestFactory factory, 
-			     CORBA_Environment *ev)
+static void
+testVariableLengthArray (test_TestFactory   factory, 
+			 CORBA_Environment *ev)
 {
   test_ArrayServer objref;
   test_StrArray inArg, inoutArg;
@@ -674,8 +715,9 @@ void testVariableLengthArray (test_TestFactory factory,
 }
 
 
-void testAnyLong (test_TestFactory factory, 
-		  CORBA_Environment *ev)
+static void
+testAnyLong (test_TestFactory   factory, 
+	     CORBA_Environment *ev)
 {
   test_AnyServer objref;
   CORBA_any inArg, inoutArg, *outArg, *retn;
@@ -727,141 +769,165 @@ void testAnyLong (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testAnyString (test_TestFactory factory, 
+static void
+testAnyString (test_TestFactory   factory, 
+	       CORBA_Environment *ev)
+{
+	test_AnyServer objref;
+	CORBA_any inArg, inoutArg, *outArg, *retn;
+
+	d_print ("Testing any with strings...\n");
+	objref = test_TestFactory_getAnyServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	inArg._type = (CORBA_TypeCode)TC_CORBA_string;
+	inArg._value = &constants_STRING_IN;
+	CORBA_any_set_release (&inArg, CORBA_FALSE);
+
+	inoutArg._type = (CORBA_TypeCode)TC_CORBA_string;
+	inoutArg._value = &constants_STRING_INOUT_IN;
+	CORBA_any_set_release (&inoutArg, CORBA_FALSE);
+  
+	retn = test_AnyServer_opAnyString (objref, &inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	g_assert (CORBA_TypeCode_equal (inArg._type, TC_CORBA_string, ev));
+	g_assert (strcmp (* (CORBA_char **)inArg._value, constants_STRING_IN) == 0);
+
+	g_assert (CORBA_TypeCode_equal (inoutArg._type, TC_CORBA_string, ev) );
+	g_assert (strcmp (* (CORBA_char **)inoutArg._value, constants_STRING_INOUT_OUT) == 0);
+
+	g_assert (CORBA_TypeCode_equal (outArg->_type, TC_CORBA_string, ev) );
+	g_assert (strcmp (* (CORBA_char **)outArg->_value, constants_STRING_OUT) == 0);
+
+	g_assert (CORBA_TypeCode_equal (retn->_type, TC_CORBA_string, ev) );
+	g_assert (strcmp (* (CORBA_char **)retn->_value, constants_STRING_RETN) == 0);
+
+	if (CORBA_any_get_release (&inArg)){
+		CORBA_free (inArg._value);
+		CORBA_Object_release ((CORBA_Object)inArg._type, ev);
+	}
+
+	if (CORBA_any_get_release (&inoutArg)){
+		CORBA_free (inoutArg._value);
+		CORBA_Object_release ((CORBA_Object)inoutArg._type, ev);
+	}
+
+	CORBA_free (outArg);
+	CORBA_free (retn);
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
+testAnyStrSeq (test_TestFactory   factory, 
+	       CORBA_Environment *ev)
+{
+	test_AnyServer objref;
+	CORBA_any     *retn;
+
+	d_print ("Testing any with string sequences ...\n");
+
+	objref = test_TestFactory_getAnyServer(factory,ev);
+	g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+	retn = test_AnyServer_opAnyStrSeq (objref, ev);
+	g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_free (retn);
+
+	CORBA_Object_release (objref, ev);
+}
+
+static void
+testAnyStruct (test_TestFactory   factory, 
+	       CORBA_Environment *ev)
+{
+	test_AnyServer objref;
+	CORBA_any inArg, inoutArg, *outArg, *retn;
+	test_VariableLengthStruct inArgStruct; 
+	test_VariableLengthStruct * inoutArgStruct;
+
+	d_print ("Testing any with structs...\n");
+
+	objref = test_TestFactory_getAnyServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	inoutArgStruct = test_VariableLengthStruct__alloc ();
+	inArgStruct.a= (CORBA_char*)constants_STRING_IN; /* const cast */
+	inArg._type = (CORBA_TypeCode)TC_test_VariableLengthStruct;
+	inArg._value = &inArgStruct;
+	CORBA_any_set_release (&inArg, CORBA_FALSE);
+
+
+	inoutArgStruct->a = CORBA_string_dup (constants_STRING_INOUT_IN);
+	inoutArg._type = (CORBA_TypeCode)TC_test_VariableLengthStruct;
+	inoutArg._value = inoutArgStruct;
+	CORBA_any_set_release (&inoutArg, CORBA_TRUE);
+
+	retn = test_AnyServer_opAnyStruct (objref, &inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  
+	g_assert (CORBA_TypeCode_equal (inArg._type, TC_test_VariableLengthStruct, ev));
+	g_assert (strcmp ((* (test_VariableLengthStruct*)inArg._value).a, constants_STRING_IN) == 0);
+
+	g_assert (CORBA_TypeCode_equal (inoutArg._type, TC_test_VariableLengthStruct, ev) );
+	g_assert (strcmp ((* (test_VariableLengthStruct*)inoutArg._value).a, constants_STRING_INOUT_OUT) == 0);
+
+	g_assert (CORBA_TypeCode_equal (outArg->_type, TC_test_VariableLengthStruct, ev) );
+	g_assert (strcmp ((* (test_VariableLengthStruct*)outArg->_value).a, constants_STRING_OUT) == 0);
+
+	g_assert (CORBA_TypeCode_equal (retn->_type, TC_test_VariableLengthStruct, ev) );
+	g_assert (strcmp ((* (test_VariableLengthStruct*)retn->_value).a, constants_STRING_RETN) == 0);
+
+
+	if (CORBA_any_get_release (&inArg)){
+		/* This shouldn't be called */
+		CORBA_free (inArg._value);
+		CORBA_Object_release ((CORBA_Object)inArg._type, ev);
+	}
+
+	if (CORBA_any_get_release (&inoutArg)){
+		CORBA_free (inoutArg._value);
+		CORBA_Object_release ((CORBA_Object)inoutArg._type, ev);
+	}
+
+	CORBA_free (outArg);
+	CORBA_free (retn);
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+}
+
+static void
+testSequenceOfAny (test_TestFactory   factory, 
 		   CORBA_Environment *ev)
 {
-  test_AnyServer objref;
-  CORBA_any inArg, inoutArg, *outArg, *retn;
+	/* this test just checks the memory management for seq of any */
 
-  d_print ("Testing any with strings...\n");
-  objref = test_TestFactory_getAnyServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	test_AnySeq anyseq;
+	int i;
 
-  inArg._type = (CORBA_TypeCode)TC_CORBA_string;
-  inArg._value = &constants_STRING_IN;
-  CORBA_any_set_release (&inArg, CORBA_FALSE);
+	d_print ("Testing Sequence of Any...\n");
 
-  inoutArg._type = (CORBA_TypeCode)TC_CORBA_string;
-  inoutArg._value = &constants_STRING_INOUT_IN;
-  CORBA_any_set_release (&inoutArg, CORBA_FALSE);
+	anyseq._buffer  = CORBA_sequence_CORBA_any_allocbuf (2);
+	anyseq._length  = 2;
+	CORBA_sequence_set_release (&anyseq, CORBA_TRUE);
   
-  retn = test_AnyServer_opAnyString (objref, &inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-
-  g_assert (CORBA_TypeCode_equal (inArg._type, TC_CORBA_string, ev));
-  g_assert (strcmp (* (CORBA_char **)inArg._value, constants_STRING_IN) == 0);
-
-  g_assert (CORBA_TypeCode_equal (inoutArg._type, TC_CORBA_string, ev) );
-  g_assert (strcmp (* (CORBA_char **)inoutArg._value, constants_STRING_INOUT_OUT) == 0);
-
-  g_assert (CORBA_TypeCode_equal (outArg->_type, TC_CORBA_string, ev) );
-  g_assert (strcmp (* (CORBA_char **)outArg->_value, constants_STRING_OUT) == 0);
-
-  g_assert (CORBA_TypeCode_equal (retn->_type, TC_CORBA_string, ev) );
-  g_assert (strcmp (* (CORBA_char **)retn->_value, constants_STRING_RETN) == 0);
-
-  if (CORBA_any_get_release (&inArg)){
-	CORBA_free (inArg._value);
-	CORBA_Object_release ((CORBA_Object)inArg._type, ev);
-  }
-
-  if (CORBA_any_get_release (&inoutArg)){
-	CORBA_free (inoutArg._value);
-	CORBA_Object_release ((CORBA_Object)inoutArg._type, ev);
-  }
-
-  CORBA_free (outArg);
-  CORBA_free (retn);
-
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	for (i = 0; i < anyseq._length; i++) {
+		anyseq._buffer [i]._type = (CORBA_TypeCode) TC_CORBA_string;
+		anyseq._buffer [i]._value = &constants_STRING_IN;
+		CORBA_any_set_release (
+			&anyseq._buffer[i], CORBA_FALSE);
+	}
+  
+	CORBA_free (anyseq._buffer);
 }
 
-void testAnyStruct (test_TestFactory factory, 
-		   CORBA_Environment *ev)
-{
-  test_AnyServer objref;
-  CORBA_any inArg, inoutArg, *outArg, *retn;
-  test_VariableLengthStruct inArgStruct; 
-  test_VariableLengthStruct * inoutArgStruct;
-
-  d_print ("Testing any with structs...\n");
-
-  objref = test_TestFactory_getAnyServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-
-  inoutArgStruct = test_VariableLengthStruct__alloc ();
-  inArgStruct.a= (CORBA_char*)constants_STRING_IN; /* const cast */
-  inArg._type = (CORBA_TypeCode)TC_test_VariableLengthStruct;
-  inArg._value = &inArgStruct;
-  CORBA_any_set_release (&inArg, CORBA_FALSE);
-
-
-  inoutArgStruct->a = CORBA_string_dup (constants_STRING_INOUT_IN);
-  inoutArg._type = (CORBA_TypeCode)TC_test_VariableLengthStruct;
-  inoutArg._value = inoutArgStruct;
-  CORBA_any_set_release (&inoutArg, CORBA_TRUE);
-
-  retn = test_AnyServer_opAnyStruct (objref, &inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-
-  
-  g_assert (CORBA_TypeCode_equal (inArg._type, TC_test_VariableLengthStruct, ev));
-  g_assert (strcmp ((* (test_VariableLengthStruct*)inArg._value).a, constants_STRING_IN) == 0);
-
-  g_assert (CORBA_TypeCode_equal (inoutArg._type, TC_test_VariableLengthStruct, ev) );
-  g_assert (strcmp ((* (test_VariableLengthStruct*)inoutArg._value).a, constants_STRING_INOUT_OUT) == 0);
-
-  g_assert (CORBA_TypeCode_equal (outArg->_type, TC_test_VariableLengthStruct, ev) );
-  g_assert (strcmp ((* (test_VariableLengthStruct*)outArg->_value).a, constants_STRING_OUT) == 0);
-
-  g_assert (CORBA_TypeCode_equal (retn->_type, TC_test_VariableLengthStruct, ev) );
-  g_assert (strcmp ((* (test_VariableLengthStruct*)retn->_value).a, constants_STRING_RETN) == 0);
-
-
-  if (CORBA_any_get_release (&inArg)){
-	/* This shouldn't be called */
-	CORBA_free (inArg._value);
-	CORBA_Object_release ((CORBA_Object)inArg._type, ev);
-  }
-
-  if (CORBA_any_get_release (&inoutArg)){
-	CORBA_free (inoutArg._value);
-	CORBA_Object_release ((CORBA_Object)inoutArg._type, ev);
-  }
-
-  CORBA_free (outArg);
-  CORBA_free (retn);
-
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-
-}
-
-void testSequenceOfAny (test_TestFactory factory, 
-		       CORBA_Environment *ev)
-{
-  /* this test just checks the memory management for seq of any */
-
-  test_AnySeq anyseq;
-  int i;
-
-  d_print ("Testing Sequence of Any...\n");
-
-  anyseq._buffer  = CORBA_sequence_CORBA_any_allocbuf (2);
-  anyseq._length  = 2;
-  CORBA_sequence_set_release (&anyseq, CORBA_TRUE);
-  
-  for (i=0;i<anyseq._length;i++){
-	anyseq._buffer[i]._type = (CORBA_TypeCode)TC_CORBA_string;
-	anyseq._buffer[i]._value = &constants_STRING_IN;
-	CORBA_any_set_release (&anyseq._buffer[i], CORBA_FALSE);
-  }
-  
-  CORBA_free (anyseq._buffer);
-}
-
-void
+static void
 testAnyException (test_TestFactory   factory, 
 		  CORBA_Environment *ev)
 {
@@ -877,7 +943,7 @@ testAnyException (test_TestFactory   factory,
 	CORBA_free (inArg);
 }
 
-void
+static void
 testTypeCode (test_TestFactory   factory, 
 	      CORBA_Environment *ev)
 {
@@ -908,57 +974,77 @@ testTypeCode (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testContext (test_TestFactory   factory, 
 	     CORBA_Environment *ev)
 {
-  test_ContextServer objref;
-  CORBA_Object inArg, inoutArg, outArg, retn;
-  CORBA_Context ctx;
+	test_ContextServer objref;
+	CORBA_Object inArg, inoutArg, outArg, retn;
+	CORBA_Context ctx;
 
-  d_print ("Testing Contexts...\n");
+	d_print ("Testing Contexts...\n");
 
-  objref = test_TestFactory_getContextServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	objref = test_TestFactory_getContextServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  inArg = inoutArg = outArg = retn = CORBA_OBJECT_NIL;
+	inArg = inoutArg = outArg = retn = CORBA_OBJECT_NIL;
 
-  CORBA_Context_create_child (CORBA_OBJECT_NIL, "Whatever", &ctx, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Context_create_child (CORBA_OBJECT_NIL, "Whatever", &ctx, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  CORBA_Context_set_one_value (ctx, "foo", "foo1", ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  CORBA_Context_set_one_value (ctx, "foo", "foo2", ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  CORBA_Context_set_one_value (ctx, "bar", "baaaa", ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Context_set_one_value (ctx, "foo", "foo1", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Context_set_one_value (ctx, "foo", "foo2", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Context_set_one_value (ctx, "bar", "baaaa", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  retn = test_ContextServer_opWithContext (
-	  objref, inArg, &inoutArg, &outArg, ctx, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	retn = test_ContextServer_opWithContext (
+		objref, inArg, &inoutArg, &outArg, ctx, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  CORBA_Object_release (retn, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  CORBA_Object_release (inArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  CORBA_Object_release (inoutArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  CORBA_Object_release (outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (retn, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (inArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (inoutArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  CORBA_Object_release ( (CORBA_Object)ctx, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release ( (CORBA_Object)ctx, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+#define SEQ_SIZE 4096
+static test_StrSeq *
+make_large_str_seq (void)
+{
+	test_StrSeq *seq = test_StrSeq__alloc ();
+	int i;
+	static const char base_data[] = "This is a longish test string it could go on for ever and ever";
+
+	seq->_buffer = test_StrSeq_allocbuf (SEQ_SIZE);
+
+	for (i = 0; i < SEQ_SIZE; i++) {
+		int len = 3 + (int) ((sizeof (base_data) - 3.0) * rand () / (RAND_MAX + 1.0));
+		seq->_buffer [i] = CORBA_string_dup (base_data);
+		seq->_buffer [i] [len] = '\0';
+	}
+
+	return seq;
+}
+
+static void
 testMisc (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
-	CORBA_char   *foo;
-	CORBA_Context ctx;
+	CORBA_char       *foo;
+	CORBA_Context     ctx;
+	test_BasicServer  objref;
 
 	d_print ("Testing Misc bits...\n");
 
@@ -969,24 +1055,31 @@ testMisc (test_TestFactory   factory,
 		g_assert (!strcmp (ev->_id, "IDL:CORBA/BAD_OPERATION:1.0"));
 		CORBA_exception_free (ev);
 	}
+	
 
-	if (!factory->adaptor_obj) {
-		test_BasicServer objref;
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (objref != CORBA_OBJECT_NIL);
+	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
-		objref = test_TestFactory_getBasicServer (factory, ev);
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
-		g_assert (objref != CORBA_OBJECT_NIL);
-		g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+	test_BasicServer_noImplement (objref, ev);
+	g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+	g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
+	CORBA_exception_free (ev);
+
+	if (!in_proc) {
+		test_StrSeq *seq;
+		
+		seq = make_large_str_seq ();
+		test_BasicServer_testLargeStringSeq (objref, seq, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		CORBA_free (seq);
 
-		test_BasicServer_noImplement (objref, ev);
-		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
-		g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
-		CORBA_exception_free (ev);
-
-		CORBA_Object_release (objref, ev);
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	}
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		
 	/* Check we are building full type data */
 	if (strcmp (TC_test_ObjectStruct->subtypes [0]->repo_id, 
@@ -1019,17 +1112,48 @@ testMisc (test_TestFactory   factory,
 	/*
 	 * dead reference check
 	 */
-	if (!factory->adaptor_obj) {
+	if (!in_proc) {
 		test_DeadReferenceObj obj;
 
 		obj = test_TestFactory_createDeadReferenceObj (factory, ev);
-
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		g_assert (obj != CORBA_OBJECT_NIL);
 
 		test_DeadReferenceObj_test (obj, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
+		CORBA_Object_release (obj, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	} else { /* Lets do some things on a de-activated ref */
+		test_BasicServer         obj;
+		PortableServer_ObjectId *oid;
+		POA_test_BasicServer     servant = {
+			NULL, &BasicServer_vepv
+		};
+
+		obj = create_object (
+			global_poa, POA_test_BasicServer__init,
+			&servant, ev);
+
+		oid = PortableServer_POA_servant_to_id (
+			global_poa, &servant, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		PortableServer_POA_deactivate_object (
+			global_poa, oid, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+		CORBA_free (oid);
+
+		test_BasicServer_opException (obj, ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, ex_CORBA_OBJECT_NOT_EXIST));
+		CORBA_exception_free (ev);
+
+		test_BasicServer_opOneWay (obj, "Foo", ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, ex_CORBA_OBJECT_NOT_EXIST));
+		CORBA_exception_free (ev);
+
 		CORBA_Object_release (obj, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	}
@@ -1122,6 +1246,10 @@ test_BasicServer_opStringA_cb (CORBA_Object          object,
 	g_assert (!strcmp (out_str, constants_STRING_OUT));
 	g_assert (!strcmp (ret_str, constants_STRING_RETN));
 
+	CORBA_free (inout_str);
+	CORBA_free (out_str);
+	CORBA_free (ret_str);
+
 	done = 1;
 }
 
@@ -1144,7 +1272,7 @@ test_BasicServer_opStringA (CORBA_Object       obj,
 }
 
 
-void
+static void
 testAsync (test_TestFactory   factory, 
 	   CORBA_Environment *ev)
 {
@@ -1188,19 +1316,148 @@ broken_cb (LINCConnection *connection, gboolean *broken)
 	*broken = TRUE;
 }
 
-void
+static void
+testPingPong (test_TestFactory   factory, 
+	      CORBA_Environment *ev)
+{
+	test_PingPongServer r_objref, l_objref, objref;
+	CORBA_unsigned_long before_remote_hash;
+	CORBA_unsigned_long after_remote_hash;
+
+	d_print ("Testing ping pong invocations ...\n");
+	r_objref = test_TestFactory_createPingPongServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	l_objref = TestFactory_createPingPongServer (NULL, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (l_objref, ev); /* only want 1 ref */
+	g_assert (ORBit_small_get_servant (l_objref) != NULL);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	before_remote_hash = CORBA_Object_hash (l_objref, 0, ev);
+
+	test_PingPongServer_pingPong (r_objref, l_objref, 64, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	d_print ("Testing ping pong reg / lookup ...\n");
+	test_PingPongServer_set (r_objref, l_objref, "Foo", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	objref = test_PingPongServer_get (r_objref, "Foo", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (ORBit_small_get_servant (objref) != NULL);
+	g_assert (l_objref == objref);
+
+	after_remote_hash = CORBA_Object_hash (l_objref, 0, ev);
+
+	d_print ("Testing hashing\n");
+	g_assert (before_remote_hash == after_remote_hash);
+
+	d_print ("Testing equivalence\n");
+	g_assert (CORBA_Object_is_equivalent (
+		l_objref, objref, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		l_objref, l_objref, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		r_objref, r_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		r_objref, l_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		l_objref, r_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		l_objref, CORBA_OBJECT_NIL, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		CORBA_OBJECT_NIL, l_objref, ev));
+
+	if (!in_proc) {
+		int i;
+		ORBitConnection *cnx = ORBit_small_get_connection (r_objref);
+		const char *base =
+			"This string is in order to provide some "
+			"more bulky data on the wire, the larger "
+			"the better. When the socket buffer fills "
+			"we start exercising the linc buffering code "
+			"and hopefully we get an exception somewhere "
+			"indicating that the buffer is full and that "
+			"the write has failed & that the connection is "
+			"now saturated ";
+		char *str;
+		gboolean broken;
+
+		g_assert (cnx != NULL);
+		ORBit_connection_set_max_buffer (cnx, 10000);
+
+		str = g_strdup (base);
+		for (i = 0; i < 4; i++) {
+			char *new;
+			new = g_strconcat (str, str, NULL);
+			g_free (str);
+			str = new;
+		}
+
+		d_print ("Testing non blocking IO ...\n");
+
+		ORBit_small_listen_for_broken (
+			r_objref, G_CALLBACK (broken_cb),
+			&broken);
+
+		for (i = 0; i < 100; i++) {
+			test_PingPongServer_opSleep (
+				r_objref, str, ev);
+			if (broken)
+				break;
+		}
+
+		g_free (str);
+
+		/* If this blows - perhaps you just have a strange
+		 * system scheduler */
+		g_assert (broken);
+		CORBA_exception_free (ev);
+	}
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (l_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (r_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
+dummy_cb (LINCConnection *connection, gboolean *invoked)
+{
+	*invoked = TRUE;
+}
+
+static void
 testSegv (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
 	d_print ("Testing Fatal invocations ...\n");
 	if (!in_proc) {
 		gboolean broken = FALSE;
+		gboolean invoked = FALSE;
+		CORBA_char *id;
 
 		g_assert (ORBit_small_listen_for_broken (
 			factory, G_CALLBACK (broken_cb), &broken) ==
-			  ORBIT_CONNECTION_CONNECTED);;
+			  ORBIT_CONNECTION_CONNECTED);
 
-		test_TestFactory_segv (factory, "do it!", ev); 
+		g_assert (ORBit_small_listen_for_broken (
+			factory, G_CALLBACK (dummy_cb), &invoked) ==
+			  ORBIT_CONNECTION_CONNECTED);
+
+		g_assert (ORBit_small_unlisten_for_broken (
+			factory, G_CALLBACK (dummy_cb)) ==
+			  ORBIT_CONNECTION_CONNECTED);
+
+		id = test_TestFactory_segv (factory, "do it!", ev); 
+#ifdef DO_HARDER_SEGV
 		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
 		g_assert (!strcmp (ev->_id, "IDL:CORBA/COMM_FAILURE:1.0"));
 		CORBA_exception_free (ev);
@@ -1208,10 +1465,19 @@ testSegv (test_TestFactory   factory,
 		g_assert (ORBit_small_get_connection_status (factory) ==
 			  ORBIT_CONNECTION_DISCONNECTED);
 		g_assert (broken);
+		g_assert (!invoked);
+#else
+		if (ORBit_small_unlisten_for_broken (
+			factory, G_CALLBACK (broken_cb)) !=
+		    ORBIT_CONNECTION_CONNECTED)
+			g_warning ("Unusual race in unlisten");
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		CORBA_free (id);
+#endif
 	}
 }
 
-void
+static void
 test_initial_references (CORBA_ORB          orb,
 			 CORBA_Environment *ev)
 {
@@ -1254,31 +1520,26 @@ run_tests (test_TestFactory   factory,
 {
 	int i;
 
-#ifdef TIMING_RUN
 	for (i = 0; i < NUM_RUNS; i++) {
-#else
-	for (i = 0; i < 1; i++) {
-#endif
 		testConst ();
 		testAttribute (factory, ev);
 		testString (factory, ev);
 		testLong (factory, ev);
 		testLongLong (factory, ev);
 		testEnum (factory, ev);
-#ifndef BIG_STUBS
 		testException (factory, ev);
-#endif
 		testFixedLengthStruct (factory, ev);
 		testVariableLengthStruct (factory, ev);
 		testCompoundStruct (factory, ev);
+		testObjectStruct (factory, ev);
 		testStructAny (factory, ev);
 		testUnboundedSequence (factory, ev);
 		testBoundedSequence (factory, ev);
 		testFixedLengthUnion (factory, ev);
 		testVariableLengthUnion (factory, ev);
-#ifndef BIG_STUBS
 		testFixedLengthArray (factory, ev);
 		testVariableLengthArray (factory, ev);
+		testAnyStrSeq (factory, ev);
 		testAnyLong (factory, ev);
 		testAnyString (factory, ev);
 		testAnyStruct (factory, ev);
@@ -1289,23 +1550,18 @@ run_tests (test_TestFactory   factory,
 		testIInterface (factory, ev);
 		testMisc (factory, ev);
 		testAsync (factory, ev);
-#endif
-
-#ifndef TIMING_RUN
+		if (!in_proc)
+			testPingPong (factory, ev);
+#if NUM_RUNS == 1
 		testSegv (factory, ev);
 #endif
 	}
 	
-#ifdef TIMING_RUN
+#if NUM_RUNS > 1
 	g_warning ("Did '%d' iterations", i);
 #endif
 }
 
-/* Ugly- but hey */
-#define _IN_CLIENT_
-#include "server.c"
-#undef  _IN_CLIENT_
-
 int
 main (int argc, char *argv [])
 {
@@ -1327,20 +1583,16 @@ main (int argc, char *argv [])
 	/* In Proc ... */
 	in_proc = TRUE;
 
-#ifndef TIMING_RUN
 	fprintf (stderr, "\n --- In proc ---\n\n\n");
-	{
-		factory = get_server (global_orb, &ev);
-		g_assert (factory->profile_list == NULL);
-		g_assert (ORBit_object_get_connection (factory) == NULL);
-		run_tests (factory, &ev);
-		CORBA_Object_release (factory, &ev);
-		g_assert (ev._major == CORBA_NO_EXCEPTION);
-		factory = CORBA_OBJECT_NIL;
-	}
-	fprintf (stderr, "\n\n --- Out of proc ---\n\n\n");
-#endif
+	factory = get_server (global_orb, &ev);
+	g_assert (factory->profile_list == NULL);
+	g_assert (ORBit_object_get_connection (factory) == NULL);
+	run_tests (factory, &ev);
+	CORBA_Object_release (factory, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+	factory = CORBA_OBJECT_NIL;
 
+	fprintf (stderr, "\n\n --- Out of proc ---\n\n\n");
 	in_proc = FALSE;
 
 	{ /* read the ior from iorfile, and swizzle to an objref*/
@@ -1367,6 +1619,11 @@ main (int argc, char *argv [])
 
 	CORBA_Object_release (factory, &ev);
 	g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+	g_warning ("released factory");
+
+	CORBA_Object_release ((CORBA_Object) global_poa, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
 	
 	CORBA_ORB_destroy (global_orb, &ev);
 	CORBA_exception_free (&ev);
